const passport = require('passport');
const ClientPasswordStrategy = require('passport-oauth2-client-password');
const BearerStrategy = require('passport-http-bearer').Strategy;

const libs = `${process.cwd()}/libs`;
const config = require(`${libs}/config`);

const User = require(`${libs}/models/user`);
const Client = require(`${libs}/models/client`);
const AccessToken = require(`${libs}/models/accessToken`);
const RefreshToken = require(`${libs}/models/refreshToken`);
const email = require(`${libs}/middlewares/email`);

passport.use(new ClientPasswordStrategy( (clientId, clientSecret, done) => {
        Client.findOne({ _id: clientId }, (err, client) => {
            if (err) {
                return done(err);
            }

            if (!client) {
                return done(null, false);
            }

            if (client.clientSecret !== clientSecret) {
                return done(null, false);
            }

            return done(null, client);
        })
    }
));

passport.use(new BearerStrategy( (accessToken, done) => {
        AccessToken.findOne({ token: accessToken }, (err, token) => {
            if (err) {
                return done(err);
            }

            if (!token) {
                return done(null, false);
            }

            if( Math.round((Date.now()-token.created)/1000) > config.get('security:tokenLife') ) {
                AccessToken.remove({ token: accessToken }, (err) => {
                    if (err) {
                        return done(err);
                    }
                });

                return done(null, false, { msg: 'Token expired' });
            }

            Usuario.findById(token.userId, function(err, usuario) {
                if (err) {
                    return done(err);
                }

                if (!usuario) {
                    return done(null, false, { msg: 'Unknown user' });
                }

                var info = { scope: '*' };
                done(null, usuario, info);
            })
        })
    }
));
